/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <package_manager.h>
#include <app_manager.h>
#include <badge.h>
#include <Eina.h>
#include "$(appName).h"
#include "data.h"

static struct model_info {
	Eina_List *pkg_list;
} s_info = {
	.pkg_list = NULL,
};

static bool _app_info_cb(app_info_h app_info, void *user_data);
static bool _create_package_info(const char *app_label, const char *app_id, const char *icon_path, bool changeable, pkginfo_s **pkginfo);
static char *_get_icon_path(const char *appid);

/**
 * @brief Initialization function for data module.
 */
void data_initialize(void)
{
	int ret = app_manager_foreach_app_info(_app_info_cb, NULL);
	if (ret != APP_MANAGER_ERROR_NONE)
		dlog_print(DLOG_ERROR, LOG_TAG, "Function app_manager_foreach_app_info() failed with error %d.", ret);
}

/**
 * @brief Finalization function for data module.
 */
void data_finalize(void)
{
	Eina_List *it = NULL;
	pkginfo_s *pkginfo = NULL;

	EINA_LIST_FOREACH(s_info.pkg_list, it, pkginfo) {
		if (!pkginfo)
			continue;

		free(pkginfo->app_label);
		free(pkginfo->app_id);
		free(pkginfo->icon_path);
		free(pkginfo);
	}

	eina_list_free(s_info.pkg_list);
}

/**
 * @brief Gets the number of applications which can be affected with badge API.
 * @return This function returns the number of applications which can be affected with badge API.
 */
int data_get_application_count(void)
{
	return (int)eina_list_count(s_info.pkg_list);
}

/**
 * @brief Gets the application's package information obtained during the data module initialization procedure.
 * @param[in] index The index of an application.
 * @param[out] pkginfo The application's package information structure corresponding to the provided index.
 * @return This function returns 'true' if the function succeeds, otherwise 'false' is returned.
 */
bool data_get_application_info(int index, pkginfo_s **pkginfo)
{
	*pkginfo = eina_list_nth(s_info.pkg_list, (unsigned int)index);

	return (*pkginfo != NULL);
}

/**
 * @brief Internal callback function invoked for each enumerated privilege name with package_info_foreach_privilege_info() function.
 * @param[in] privilege_name The privilege name added to the tizen manifest file.
 * @param[in] user_data The user data passed to the package_info_foreach_privilege_info() function.
 * @return This function returns 'true' to continue the enumeration by package_info_foreach_privilege_info(),
 * otherwise 'false' is returned.
 */
static bool _package_info_privilege_info_cb(const char *privilege_name, void *user_data)
{
	static char *notification_privilege = "http://tizen.org/privilege/notification";

	bool *notification_privilege_exists = (bool*)user_data;

	if (strcmp(privilege_name, notification_privilege) == 0) {
		*notification_privilege_exists = true;
		return false;
	}

	return true;
}

/**
 * @brief Internal callback function invoked for each enumerated application with app_manager_foreach_app_info() function.
 * Within this function, the external applications are filtered which badge's may be modified by this application.
 * The external applications eligible for badge manipulation are those which:
 * - may be displayed on the Homescreen in form of an icon,
 * - is signed with the same certificate as calling application,
 * - has notification privilege added to the tizen-manifest.xml file.
 * @param[in] app_info The application information structure's handle.
 * @param[in] user_data The user data passed to the app_manager_foreach_app_info() function.
 * @return This function returns 'true' to continue the enumeration by app_manager_foreach_app_info(),
 * otherwise 'false' is returned.
 */
static bool _app_info_cb(app_info_h app_info, void *user_data)
{
	int ret;
	char *app_label = NULL;
	char *app_id = NULL;
	char *icon_path = NULL;
	char *package = NULL;
	bool privilege_exists = false;
	bool nodisplay = true;
	bool changeable;
	pkginfo_s *pkginfo = NULL;
	package_info_h package_info = NULL;
	package_manager_compare_result_type_e cmp_res = PACKAGE_MANAGER_COMPARE_MISMATCH;

	ret = app_info_get_label(app_info, &app_label);
	if (ret != APP_MANAGER_ERROR_NONE) {
		dlog_print(DLOG_ERROR, LOG_TAG, "Function app_info_get_label() failed with error %d.", ret);
		return true;
	}

	ret = app_info_is_nodisplay(app_info, &nodisplay);
	if (ret != APP_MANAGER_ERROR_NONE) {
		dlog_print(DLOG_ERROR, LOG_TAG, "Function app_info_is_nodisplay() failed with error %d.", ret);
		free(app_label);
		return true;
	}

	if (nodisplay) {
		free(app_label);
		return true;
	}

	ret = app_info_get_app_id(app_info, &app_id);
	if (ret != APP_MANAGER_ERROR_NONE) {
		dlog_print(DLOG_ERROR, LOG_TAG, "Function app_info_get_app_id() failed with error %d.", ret);
		free(app_label);
		return true;
	}

	ret = package_manager_compare_app_cert_info(PACKAGE, app_id, &cmp_res);
	if (ret != PACKAGE_MANAGER_ERROR_NONE) {
		dlog_print(DLOG_ERROR, LOG_TAG, "Function package_manager_compare_app_cert_info() failed with error %d.", ret);
		free(app_label);
		free(app_id);
		return true;
	}

	ret = app_info_get_package(app_info, &package);
	if (ret != APP_MANAGER_ERROR_NONE)
	{
		dlog_print(DLOG_ERROR, LOG_TAG, "Function app_info_get_package() failed with error %d.", ret);
		free(app_label);
		free(app_id);
		return true;
	}

	ret = package_info_create(package, &package_info);
	if (ret != PACKAGE_MANAGER_ERROR_NONE)
	{
		dlog_print(DLOG_ERROR, LOG_TAG, "Function package_info_create() failed with error %d.", ret);
		free(app_label);
		free(app_id);
		free(package);
		return true;
	}

	ret = package_info_foreach_privilege_info(package_info, _package_info_privilege_info_cb, &privilege_exists);
	if (ret != PACKAGE_MANAGER_ERROR_NONE)
	{
		dlog_print(DLOG_ERROR, LOG_TAG, "Function package_info_foreach_privilege_info() failed with error %d.", ret);
		free(app_label);
		free(app_id);
		free(package);
		package_info_destroy(package_info);
		return true;
	}

	changeable = (cmp_res == PACKAGE_MANAGER_COMPARE_MATCH) && privilege_exists;
	icon_path = _get_icon_path(app_id);

	if (_create_package_info(app_label, app_id, icon_path, changeable, &pkginfo))
		s_info.pkg_list = eina_list_append(s_info.pkg_list, (void *)pkginfo);

	dlog_print(DLOG_INFO, LOG_TAG, "Application found: %s.", app_id);

	free(app_id);
	free(app_label);
	free(icon_path);
	free(package);
	package_info_destroy(package_info);

	return true;
}

/**
 * @brief Internal function responsible for package information structure creation.
 * The package information structure is internally defined and stores the identifier
 * and package name of an external application enumerated found by app_manager_foreach_app_info() function.
 * @param[in] pkg_name The package name of an external application.
 * @param[in] app_id The identifier of an external application.
 * @param[in] icon_path The absolute path to the application's icon file.
 * @param[in] changeable The flag denoting if an application's badge may be manipulated.
 * @return This function returns 'true' if a package structure was successfully created,
 * otherwise 'false' is returned.
 */
static bool _create_package_info(const char *app_label, const char *app_id, const char *icon_path, bool changeable, pkginfo_s **pkginfo)
{
	if (!app_label || !app_id) {
		dlog_print(DLOG_ERROR, LOG_TAG, "_create_package_info() failed. Wrong input arguments.");
		return false;
	}

	*pkginfo = (pkginfo_s *)malloc(sizeof(pkginfo_s));
	if (!*pkginfo) {
		dlog_print(DLOG_ERROR, LOG_TAG, "Memory allocation failed for pkginfo_s.");
		return false;
	}

	(*pkginfo)->app_label = strdup(app_label);
	(*pkginfo)->app_id = strdup(app_id);
	(*pkginfo)->changeable = changeable;
	(*pkginfo)->badge_count = 0;

	if (icon_path)
		(*pkginfo)->icon_path = strdup(icon_path);

	return true;
}

/**
 * @brief Function obtains the absolute path to the application's icon file.
 * @param[out] appid The appid of the application which icon's path has to be obtained.
 * @return The path to the application's icon file if the function succeeds,
 * or NULL if the function fails.
 */
static char *_get_icon_path(const char *appid)
{
	int ret;
	app_info_h app_info = NULL;
	char *icon_path = NULL;

	ret = app_info_create(appid, &app_info);
	if (ret != APP_MANAGER_ERROR_NONE) {
		dlog_print(DLOG_ERROR, LOG_TAG, "app_info_create() failed. Err = %d.", ret);
		return NULL;
	}

	ret = app_info_get_icon(app_info, &icon_path);
	if (ret != APP_MANAGER_ERROR_NONE)
		dlog_print(DLOG_ERROR, LOG_TAG, "app_info_get_icon() failed. Err = %d.", ret);

	app_info_destroy(app_info);

	return (ret == APP_MANAGER_ERROR_NONE ? icon_path : NULL);
}
