/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <Elementary.h>
#include <tizen.h>
#include <efl_extension.h>
#include "$(appName).h"
#include "view_defines.h"
#include "view.h"

static struct view_info {
	Evas_Object *win;
	Evas_Object *conform;
	Evas_Object *layout_main;
	Evas_Object *layout_app_name;
	Evas_Object *layout_badge_counter;
	Evas_Object *apps_list;
	Elm_Object_Item *item_selected;
	int badge_counter_value;
	update_badge_request_cb update_badge_request_func;
	get_badge_request_cb get_badge_request_func;
} s_info = {
	.win = NULL,
	.conform = NULL,
	.layout_main = NULL,
	.layout_app_name = NULL,
	.layout_badge_counter = NULL,
	.apps_list = NULL,
	.item_selected = NULL,
	.badge_counter_value = 0,
	.update_badge_request_func = NULL,
	.get_badge_request_func = NULL,
};

static void _win_delete_request_cb(void *data, Evas_Object *obj, void *event_info);
static void _layout_back_cb(void *data, Evas_Object *obj, void *event_info);
static Evas_Object *_get_item_content_cb(void *data, Evas_Object *obj, const char *part);
static void _badge_counter_updated_cb(void *data, Evas_Object *obj, Edje_Message_Type type, int id, void *msg);
static void _badge_apply_clicked_cb(void *data, Evas_Object *obj, const char *emission, const char *source);
static void _apps_list_item_selected_cb(void *data, Evas_Object *obj, void *event_info);
static void _popup_timeout_cb(void *data, Evas_Object *obj, void *event_info);
static void _get_app_resource(const char *edj_file_in, char *edj_path_out);
static bool _create_main_layout(Evas_Object *parent, const char *part_name);
static void _display_app_name(const char *app_name);
static void _display_badge_counter(int badge_counter);
static void _toggle_app_name_hint_visibility(bool state);
static void _set_image_grayscale(Evas_Object *image);
static void _display_toast(const char *app_name);
static void _set_item_highlight(Elm_Object_Item *item, bool highlighted);

/**
 * @brief Creates essential objects: window, conformant and layout.
 * @param[in] user_data The data passed by the user.
 * @return This function returns EINA_TRUE if the entire view was created successfully,
 * otherwise EINA_FALSE is returned.
 */
Eina_Bool view_create(void *user_data)
{
	/* Create the window */
	s_info.win = view_create_win(PACKAGE);
	if (s_info.win == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "failed to create a window.");
		return EINA_FALSE;
	}

	/* Create the conformant */
	s_info.conform = view_create_conformant(s_info.win);
	if (s_info.conform == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "failed to create a conformant");
		return EINA_FALSE;
	}

	if (!_create_main_layout(s_info.conform, "default"))
		return EINA_FALSE;

	/* Show the window after main view is set up */
	evas_object_show(s_info.win);

	return EINA_TRUE;
}

/**
 * @brief Creates a basic window named with package name.
 * @param[in] pkg_name Name of the window.
 * @return This function returns the window object if it was created successfully,
 * otherwise NULL is returned.
 */
Evas_Object *view_create_win(const char *pkg_name)
{
	Evas_Object *win = NULL;

	/*
	 * Window
	 * Create and initialize elm_win.
	 * elm_win is mandatory to manipulate the window.
	 */
	win = elm_win_util_standard_add(pkg_name, pkg_name);
	elm_win_conformant_set(win, EINA_TRUE);
	elm_win_indicator_mode_set(win, ELM_WIN_INDICATOR_SHOW);
	elm_win_indicator_opacity_set(win, ELM_WIN_INDICATOR_OPAQUE);
	elm_win_autodel_set(win, EINA_TRUE);

	evas_object_smart_callback_add(win, "delete,request", _win_delete_request_cb, NULL);

	return win;
}

/**
 * @brief Creates a conformant object.
 * @param[in] win The object to which you want to set the conformant.
 * Conformant is mandatory for base GUI to have proper size.
 * @return This function returns the conformant object if it was created successfully,
 * otherwise NULL is returned.
 */
Evas_Object *view_create_conformant(Evas_Object *win)
{
	/*
	 * Conformant
	 * Create and initialize elm_conformant.
	 * elm_conformant is mandatory for base GUI to have proper size
	 * when indicator or virtual keypad is visible.
	 */
	Evas_Object *conform = NULL;

	if (win == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "window is NULL.");
		return NULL;
	}

	conform = elm_conformant_add(win);
	evas_object_size_hint_weight_set(conform, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	elm_win_resize_object_add(win, conform);

	evas_object_show(conform);
	elm_object_signal_emit(conform, "elm,state,indicator,overlap", "elm");

	return conform;
}

/**
 * @brief Creates a layout object for parent based on provided EDJE script.
 * @param[in] parent The parent object for the layout.
 * @param[in] edj_file_name The relative path to the layout EDJE script.
 * @param[in] edj_group The name of the group to be loaded from the EDJE script.
 * @param[in] part_name The name of the parent's part where the newly created layout object
 * is to be set. If this argument is NULL, then the newly created layout object is not set anywhere.
 * @return The function returns layout object if it was created successfully,
 * otherwise NULL is returned.
 */
Evas_Object *view_create_layout(Evas_Object *parent, const char *edj_file_name, const char *edj_group, const char *part_name)
{
	char edj_path[PATH_MAX] = {0, };
	Evas_Object *layout;

	if (!parent || !edj_file_name || !edj_group) {
		dlog_print(DLOG_ERROR, LOG_TAG, "Wrong input arguments.");
		return NULL;
	}

	_get_app_resource(edj_file_name, edj_path);

	layout = elm_layout_add(parent);
	if (!layout) {
		dlog_print(DLOG_ERROR, LOG_TAG, "Function elm_layout_add() failed.");
		return NULL;
	}

	if (!elm_layout_file_set(layout, edj_path, edj_group)) {
		dlog_print(DLOG_ERROR, LOG_TAG, "Function elm_layout_file_set() failed.");
		evas_object_del(layout);
		return NULL;
	}

	evas_object_size_hint_weight_set(layout, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);

	if (part_name)
		elm_object_part_content_set(parent, part_name, layout);

	return layout;
}

/**
 * @brief Creates an image object with image file set.
 * In order to set the image file, the elm_image_file_set() function must be used.
 * @param[in] parent The parent object for image object.
 * @param[in] file_path The absolute path to the image file to be displayed.
 * @return The function returns image object if it was created successfully,
 * otherwise 'NULL' is returned.
 */
Evas_Object *view_create_image(Evas_Object *parent, const char *file_path)
{
	Evas_Object *image = elm_image_add(parent);
	if (!image) {
		dlog_print(DLOG_ERROR, LOG_TAG, "Function elm_image_add() failed.");
		return NULL;
	}

	elm_image_smooth_set(image, EINA_TRUE);
	elm_image_aspect_fixed_set(image, EINA_TRUE);

	if (file_path)
		elm_image_file_set(image, file_path, NULL);

	return image;
}

/**
 * @brief Creates popup window with 'toast' style applied.
 * @param[in] parent The parent object for toast popup window object.
 * @param[in] text The text to be displayed in the toast popup window.
 * @param[in] timeout The toast popup window display timeout.
 * @return Function returns toast popup window object if it was successfully created,
 * otherwise NULL is returned.
 */
Evas_Object *view_create_toast(Evas_Object *parent, const char *text, double timeout)
{
	Evas_Object *popup = elm_popup_add(parent);
	if (!popup) {
		dlog_print(DLOG_ERROR, LOG_TAG, "Function elm_popup_add() failed.");
		return NULL;
	}

	elm_object_style_set(popup, "toast");
	evas_object_size_hint_weight_set(popup, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	elm_object_text_set(popup, text);
	elm_popup_timeout_set(popup, timeout);
	evas_object_smart_callback_add(popup, "timeout", _popup_timeout_cb, popup);

	return popup;
}

/**
 * @brief Creates a genlist object and sets it to the parent object.
 * @param[in] parent The parent object for the genlist.
 * @param[in] part_name The name of the EDJE part where the genlist is to be set.
 * @return The function returns genlist object if it was created successfully,
 * otherwise NULL is returned.
 */
Evas_Object *view_create_genlist(Evas_Object *parent, const char *part_name)
{
	Evas_Object *list;

	if (!parent || !part_name) {
		dlog_print(DLOG_ERROR, LOG_TAG, "view_create_genlist() failed. Wrong input arguments.");
		return NULL;
	}

	list = elm_genlist_add(parent);
	if (!list) {
		dlog_print(DLOG_ERROR, LOG_TAG, "elm_genlist_add() failed.");
		return NULL;
	}

	evas_object_size_hint_weight_set(list, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	elm_object_part_content_set(parent, part_name, list);

	evas_object_show(list);

	return list;
}

/**
 * @brief Creates an item class for genlist object.
 * @param[in] on_text_get_cb The callback function's handler invoked when the genlist's item has to be redrawn.
 * @return The function returns item class if it was created successfully,
 * otherwise 'NULL' is returned.
 */
Elm_Genlist_Item_Class *view_create_genlist_item_class(Elm_Genlist_Item_Content_Get_Cb on_content_get_cb)
{
	static Elm_Genlist_Item_Class *itc = NULL;

	if (!itc) {
		itc = elm_genlist_item_class_new();
		if (!itc) {
			dlog_print(DLOG_ERROR, LOG_TAG, "Function elm_genlist_item_class_new() failed.");
			return NULL;
		}

		itc->item_style = "full";
		itc->func.text_get = NULL;
		itc->func.content_get = on_content_get_cb;
		itc->func.state_get = NULL;
		itc->func.del = NULL;
	}

	return itc;
}

/**
 * @brief Function sets view callbacks.
 * @param[in] update_badge_request_func The callback function to be invoked
 * when the 'Update' button was pressed.
 * @param[in] get_badge_request_func The callback function to be invoked
 * whenever the view module needs to acquire valid badge counter value from an external application.
 */
void view_set_callbacks(update_badge_request_cb update_badge_request_func, get_badge_request_cb get_badge_request_func)
{
	s_info.update_badge_request_func = update_badge_request_func;
	s_info.get_badge_request_func = get_badge_request_func;
}

/**
 * @brief Function adds an eligible application to the display list.
 * @param[in] pkginfo The structure containing required application's information.
 * @return The function returns 'true' if the application was successfully added to the list,
 * otherwise 'false' is returned.
 */
bool view_add_application(pkginfo_s *pkginfo)
{
	Elm_Genlist_Item_Class *itc = view_create_genlist_item_class(_get_item_content_cb);
	Elm_Object_Item *item;

	if (!pkginfo) {
		dlog_print(DLOG_ERROR, LOG_TAG, "view_add_application() failed. Wrong input arguments.");
		return false;
	}

	item = elm_genlist_item_append(s_info.apps_list, itc, (void*)pkginfo, NULL, ELM_GENLIST_ITEM_NONE, _apps_list_item_selected_cb, (void*)pkginfo);
	if (!item) {
		dlog_print(DLOG_ERROR, LOG_TAG, "elm_genlist_item_append() failed.");
		return false;
	}

	dlog_print(DLOG_INFO, LOG_TAG, "Application added to list: %s. Changeable: %d.", pkginfo->app_id, (int)pkginfo->changeable);

	return true;
}

/**
 * @brief Destroys window and frees its resources.
 */
void view_destroy(void)
{
	if (s_info.win == NULL)
		return;

	evas_object_del(s_info.win);
}

/**
 * @brief Internal callback function invoked when the main window needs to be destroyed.
 * @param[in] data The user data passed to the evas_object_smart_callback_add() function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] event_info The structure containing the information on this event.
 */
static void _win_delete_request_cb(void *data, Evas_Object *obj, void *event_info)
{
	ui_app_exit();
}

/**
 * @brief Internal callback function invoked on HW Back button press.
 * @param[in] data The user data passed to the eext_object_event_callback_add() function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] event_info The structure containing the information on this event.
 */
static void _layout_back_cb(void *data, Evas_Object *obj, void *event_info)
{
	/* Let window go to hide state. */
	elm_win_lower(s_info.win);
}

/**
 * @brief Internal callback function invoked on item selection from the applications list.
 * @param[in] data The user data passed to the evas_object_smart_callback_add() function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] event_info The structure containing the information on this event.
 */
static void _apps_list_item_selected_cb(void *data, Evas_Object *obj, void *event_info)
{
	pkginfo_s *pkginfo = (pkginfo_s *)data;

	if (!pkginfo->changeable)
		return;

	elm_genlist_item_selected_set((Elm_Object_Item*)event_info, EINA_FALSE);
	_set_item_highlight(s_info.item_selected, false);

	s_info.item_selected = (Elm_Object_Item*)event_info;
	if (!s_info.item_selected) {
		dlog_print(DLOG_ERROR, LOG_TAG, "_apps_list_item_selected_cb() failed. No item is selected.");
		return;
	}

	_set_item_highlight(s_info.item_selected, true);
	_display_app_name(pkginfo->app_label);
	_display_badge_counter(pkginfo->badge_count);
	_toggle_app_name_hint_visibility(false);
}

/**
 * @brief Internal callback function invoked on toast popup window timeout.
 * This callback function destroys the popup window object.
 * @param[in] data The user data passed to the evas_object_smart_callback_add() function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] event_info The structure containing the information on this event.
 */
static void _popup_timeout_cb(void *data, Evas_Object *obj, void *event_info)
{
	evas_object_del((Evas_Object *)data);
}

/**
 * @brief Internal callback function invoked on list's item display.
 * This function is assigned to the func.content_get handler of genlist item's class structure.
 * It is responsible for items layout creation and its text part population
 * with provided text content.
 * @param[in] data The user data passed to the elm_genlist_item_append() function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] event_info The structure containing the information on this event.
 * @return The layout object to be displayed in item's content area.
 */
static Evas_Object* _get_item_content_cb(void *data, Evas_Object *obj, const char *part)
{
	pkginfo_s *pkginfo = (pkginfo_s*)data;
	Evas_Object *layout;
	Evas_Object *image;
	char str[4] = "99+";

	if (!pkginfo || !s_info.get_badge_request_func) {
		dlog_print(DLOG_ERROR, LOG_TAG, "_get_item_content_cb() failed. Invalid input argument.");
		return NULL;
	}

	layout = view_create_layout(s_info.apps_list, EDJ_APPS_LIST_ITEM, GRP_APPS_LIST_ITEM, NULL);
	if (!layout)
		return NULL;

	image = view_create_image(obj, pkginfo->icon_path);
	if (image)
		elm_object_part_content_set(layout, PART_APPS_LIST_ITEM_ICON, image);

	elm_object_part_text_set(layout, PART_APPS_LIST_ITEM_NAME, pkginfo->app_label);

	if (pkginfo->changeable) {
		s_info.get_badge_request_func(&pkginfo);

		if (pkginfo->badge_count < 100)
			snprintf(str, sizeof(str), "%d", pkginfo->badge_count);

		elm_object_part_text_set(layout, PART_APPS_LIST_ITEM_BADGE_COUNTER, str);
		elm_layout_signal_emit(layout, SIGNAL_APPS_LIST_ITEM_ENABLE, "");
	} else {
		_set_image_grayscale(image);
	}

	return layout;
}

/**
 * @brief Internal callback function which is responsible for received EDJE message processing.
 * If the message of type EDJE_MESSAGE_INT with identifier MSG_ID_BADGE_COUNTER_VALUE,
 * containing 1 argument is received then badge counter value change is notified.
 * @param[in] data The user data passed to the attachment function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] type The type of the message received.
 * @param[in] id The message identifier.
 * @param[in] msg The received message.
 */
static void _badge_counter_updated_cb(void *data, Evas_Object *obj, Edje_Message_Type type, int id, void *msg)
{
	Edje_Message_Int *message = (Edje_Message_Int *)msg;

	if (type != EDJE_MESSAGE_INT) {
		dlog_print(DLOG_ERROR, LOG_TAG, "_badge_counter_updated_cb() failed. Not handled message type received.");
		return;
	} else if (id != MSG_ID_BADGE_COUNTER_CHANGED) {
		dlog_print(DLOG_ERROR, LOG_TAG, "_badge_counter_updated_cb() failed. Not handled message id received.");
		return;
	}

	s_info.badge_counter_value = message->val;
	dlog_print(DLOG_INFO, LOG_TAG, "Badge counter value: %d.", s_info.badge_counter_value);
}

/**
 * @brief Internal callback function invoked on "Apply" button click.
 * @param[in] data The user data passed to the callback attachment function.
 * @param[in] obj The object emitting the signal resulting with this callback function invocation.
 * @param[in] emission The emitted signal name.
 * @param[in] source The name of the part emitting the signal.
 */
static void _badge_apply_clicked_cb(void *data, Evas_Object *obj, const char *emission, const char *source)
{
	pkginfo_s *pkginfo;

	if (!s_info.update_badge_request_func) {
		dlog_print(DLOG_ERROR, LOG_TAG, "_badge_apply_clicked_cb() failed. Badge update request handler is not assigned.");
		return;
	}

	if (!s_info.item_selected) {
		dlog_print(DLOG_ERROR, LOG_TAG, "Select an application to manipulate its badge.");
		return;
	}

	pkginfo = (pkginfo_s*)elm_object_item_data_get(s_info.item_selected);
	if (!pkginfo) {
		dlog_print(DLOG_ERROR, LOG_TAG, "Function elm_object_item_data_get() failed.");
		return;
	}

	if (s_info.update_badge_request_func(pkginfo, s_info.badge_counter_value)) {
		pkginfo->badge_count = s_info.badge_counter_value;
		_display_toast(pkginfo->app_label);
		dlog_print(DLOG_INFO, LOG_TAG, "Badge counter value for '%s' applied.", pkginfo->app_id);
	} else {
		dlog_print(DLOG_ERROR, LOG_TAG, "Badge counter value for '%s' application error.", pkginfo->app_id);
	}

	elm_genlist_item_update(s_info.item_selected);
}

/**
 * @brief Internal function which creates fully qualified path to the provided resource file.
 * @param[in] edj_file_in The file name.
 * @param[out] edj_path_out The fully qualified path to the edj_file_in file.
 */
static void _get_app_resource(const char *edj_file_in, char *edj_path_out)
{
	char *res_path = app_get_resource_path();
	if (res_path) {
		snprintf(edj_path_out, PATH_MAX, "%s%s", res_path, edj_file_in);
		free(res_path);
	}
}

/**
 * @brief Internal function creates the main application's layout.
 * @param[in] parent The parent object for the main layout.
 * @param[in] part_name The name of the part within parent object where the
 * main layout is to be set.
 * @return Function returns 'true' if the main layout was created successfully,
 * otherwise 'false' is returned.
 */
static bool _create_main_layout(Evas_Object *parent, const char *part_name)
{
	Evas_Object *layout;

	s_info.layout_main = view_create_layout(s_info.conform, EDJ_MAIN, GRP_MAIN, "default");
	if (!s_info.layout_main)
		return false;

	eext_object_event_callback_add(s_info.layout_main, EEXT_CALLBACK_BACK, _layout_back_cb, NULL);

	s_info.layout_app_name = view_create_layout(s_info.layout_main, EDJ_PANELS, GRP_PANEL_APP_NAME, PART_MAIN_APP_NAME);
	if (!s_info.layout_app_name)
		return false;

	s_info.layout_badge_counter = view_create_layout(s_info.layout_main, EDJ_PANELS, GRP_PANEL_BADGE_COUNTER, PART_MAIN_BADGE_COUNTER);
	if (!s_info.layout_badge_counter)
		return false;

	edje_object_message_handler_set(elm_layout_edje_get(s_info.layout_badge_counter), _badge_counter_updated_cb, NULL);

	layout = view_create_layout(s_info.layout_main, EDJ_PANELS, GRP_PANEL_BADGE_APPLY, PART_MAIN_BADGE_APPLY);
	if (!layout)
		return false;

	elm_layout_signal_callback_add(layout, SIGNAL_BADGE_APPLY_CLICKED, PART_PANEL_BADGE_APPLY, _badge_apply_clicked_cb, NULL);

	layout = view_create_layout(s_info.layout_main, EDJ_PANELS, GRP_PANEL_APPS_LIST, PART_MAIN_APPS_LIST);
	if (!layout)
		return false;

	s_info.apps_list = view_create_genlist(layout, PART_PANEL_APPS_LIST);
	if (!s_info.apps_list)
		return false;

	return true;
}

/**
 * @brief Internal function which displays the application name.
 * @param[in] app_name The name of the application to be displayed.
 */
static void _display_app_name(const char *app_name)
{
	elm_object_part_text_set(s_info.layout_app_name, PART_PANEL_APP_NAME_CAPTION, app_name);
}

/**
 * @brief Internal function which displays the badge counter value.
 * @param[in] badge_counter The value to be displayed.
 */
static void _display_badge_counter(int badge_counter)
{
	Edje_Message_Int msg = { .val = badge_counter, };

	edje_object_message_send(elm_layout_edje_get(s_info.layout_badge_counter), EDJE_MESSAGE_INT, MSG_ID_BADGE_COUNTER_SET, (void *)&msg);
}

/**
 * @brief Internal function which changes the visibility state of the application name hint label.
 * @param[in] state Set to 'true' to show the hint label or 'false' to hide it.
 */
static void _toggle_app_name_hint_visibility(bool state)
{
	if (state)
		elm_layout_signal_emit(s_info.layout_app_name, SIGNAL_APP_NAME_HINT_SHOW, "");
	else
		elm_layout_signal_emit(s_info.layout_app_name, SIGNAL_APP_NAME_HINT_HIDE, "");
}

/**
 * @brief Internal function which changes the image color to gray scale.
 * @param[in] image The target image to be updated.
 */
static void _set_image_grayscale(Evas_Object *image)
{
	int i;
	int width = 0;
	int height = 0;
	unsigned int *buff;
	unsigned int mean;

	elm_image_object_size_get(image, &width, &height);
	buff = evas_object_image_data_get(elm_image_object_get(image), EINA_TRUE);
	if (!buff) {
		dlog_print(DLOG_ERROR, LOG_TAG, "Function evas_object_image_data_get() failed.");
		return;
	}

	for (i = 0; i < width * height; i++) {
		if ((buff[i] & 0x00ffffff) == 0x00)
			continue;

		mean = (((buff[i] & 0x00ff0000) >> 16) + ((buff[i] & 0x0000ff00) >> 8) + (buff[i] & 0x000000ff)) / 3;
		buff[i] = (buff[i] & 0xff000000) + (mean << 16) + (mean << 8) + mean;
	}

	evas_object_image_data_set(image, (void *)buff);
}

/**
 * @brief Internal function which displays a toast message with provided text content.
 * @param[in] app_name The name of the application which is included in displayed message.
 */
static void _display_toast(const char *app_name)
{
	Evas_Object *toast;
	char msg[256] = {0,};

	snprintf(msg, sizeof(msg), "Badge updated.<br>Check the Homescreen for<br><b>%s</b><br>application's icon", app_name);

	toast = view_create_toast(s_info.win, msg, 3.0);
	evas_object_show(toast);
}

/**
 * @brief Internal function which sets the highlight state to an applications list item.
 * @param[in] item The target list item.
 * @param[in] highlighted The state of the highlight.
 */
static void _set_item_highlight(Elm_Object_Item *item, bool highlighted)
{
	Evas_Object *layout;

	if (!item)
		return;

	layout = elm_object_item_part_content_get(item, "elm.swallow.content");

	if (!layout)
		return;

	if (highlighted)
		elm_layout_signal_emit(layout, SIGNAL_APPS_LIST_ITEM_SET_HIGHLIGHT, "");
	else
		elm_layout_signal_emit(layout, SIGNAL_APPS_LIST_ITEM_UNSET_HIGHLIGHT, "");
}
